require 'fastercsv'


class Map < ActiveRecord::Base
  def self.from_city_name(city_name)
    self.find_or_initialize_by_name(city_name).save!
  end

  def self.pair
    if Mash.count == 0
      self.all(:order => 'RANDOM()', :limit => count)
    else
      first_conditions = <<-SQL
      NOT EXISTS (
          SELECT * FROM `mashes`
          WHERE `winner` = `maps`.`id`
      )
      SQL
      second_conditions = <<-SQL
      `maps`.`id` != :first_id
          AND (
              SELECT COUNT(*) FROM `mashes`
              WHERE `winner` = :first_id
          ) = (
              SELECT COUNT(*) FROM `mashes`
              WHERE `winner` = `maps`.`id`
          )
      SQL

      first = self.all(
          :order => 'RANDOM()',
          :limit => 1,
          :conditions => [first_conditions]
      ).first
      second = self.all(
          :order => 'RANDOM()',
          :limit => 1,
          :conditions => [second_conditions, {:first_id => first.id}]
      ).first
      [first, second]
    end
  end

  def self.import(csv_filename)
    FasterCSV.parse(open(csv_filename), :headers => true,
        :header_converters => [:downcase, :symbol]).each do |row|
      map = self.find_or_initialize_by_name(
        "#{row[:city]}, #{row[:country]}"
      )
      map.save
      if block_given?
        yield map
      end
    end
  end

  def >(other)
    self.points > other.points
  end

  def <(other)
    self.points < other.points
  end

  def ==(other)
    self.points == other.points
  end

  def rounds
    Mash.count(:conditions => ['map_a = :id OR map_b = :id', {:id => self.id}])
  end

  def points
    Mash.count(:conditions => ['winner = ?', self.id])
  end
end
